import { Component, OnInit } from '@angular/core';
import { MorseService } from '../morse.service';
import { AngularFireDatabase } from 'angularfire2/database';
import { ActivatedRoute } from '@angular/router';

@Component({
  selector: 'app-game',
  templateUrl: './game.component.html',
  styleUrls: ['./game.component.css']
})
export class GameComponent implements OnInit {

  stockPhrases = [
    'How do I tweet with that?',
    'Now I see why they use OMG, WRU, WRK etc.',
    'Let see, where is the upload button?',
    'The telegraph is way better than smoke signals.',
    'No internet - no funny cat videos!',
    'Oh no I should learn the morse code now!'
  ];

  statusText = 'wait';

  isSender = true;
  nextLetter = '';
  bulbOn = false;
  realPhrase = '';
  guessedPart = '_';
  phraseToGuess = '';
  errors = 0;
  curLetter = -1;
  showMorseTokenIdx = 0;
  morseCode: Array<number>;
  timeInterval = 0.2*1000; // in seconds

  currentMorseTyped = '';
  currentMorseHint = 'wait';

  lastDownEvent = 0;

  bulbOnTimer = null;
  bulbOffTimer = null;

  localMode = true;
  remoteData = null;
  remotePhraseData = null;
  alreadySent = false;
  dotSound;
  dashSound;

  constructor(private morseService: MorseService, private db: AngularFireDatabase, private route: ActivatedRoute) { }

  ngOnInit() {
    this.route.params.subscribe((params) => {
      this.isSender = params.sender === 'sender1234';
      
      this.remoteData = this.db.object('/curPhrase');
      this.remoteData.valueChanges().subscribe( (curPhrase) => {
        console.log(curPhrase);
        if(!curPhrase || params.sender === 'local') {
          console.error('Missing firebase data!');
          console.log('Switching to local mode');
          this.localMode = true;
          this.remotePhraseData = {
            text: this.getPhraseByIdx(params.phraseIdx),
            senderErrors: 100 //we start with a large number
          };
          setTimeout(() => this.startGuessing(), 2000);
        }
        else {
          this.localMode = false;
          this.remotePhraseData = curPhrase;
          if(this.remoteData && this.isSender && !this.alreadySent) {
            this.alreadySent = true;
            this.remoteData.set({
              text: this.getPhraseByIdx(params.phraseIdx),
              senderErrors: 100 //we start with a large number
            });
          }
          setTimeout(() => this.startGuessing(), 2000);
        }        
      }, (err) => {
        console.error('Can not read the firebase data!');
        console.log('Switching to local mode');
        this.localMode = true;
        this.remotePhraseData = {
          text: this.getPhraseByIdx(params.phraseIdx),
          senderErrors: 100 //we start with a large number
        };
        setTimeout(() => this.startGuessing(), 2000);
      });
    });    
  }

  getPhraseByIdx(idx): string {
    if(!this.stockPhrases[idx]) {
      return this.stockPhrases[0];
    }
    
    return this.stockPhrases[idx];
  }

  validatePhrase() {
    //we replace low dash with a dash so there are no any confusions
    //the morseService will return space for ecery unrecognized character
    this.realPhrase = this.realPhrase.replace(/_/g, '-');
  }

  isNextSpace(): boolean {
    // shows if the next letter is space
    return this.morseCode.length === 7;
  }

  showLetter() {
    this.currentMorseTyped = '';
    this.morseCode = this.morseService.getMorseForLetter(this.realPhrase.charAt(this.curLetter));
    if (this.isNextSpace()) {
      this.statusText = 'break';
      this.currentMorseHint = '';
      this.bulbOff();
      return;
    }

    if(this.isSender) {
      this.currentMorseHint = this.morseService.getMorseStringForLetter(this.realPhrase.charAt(this.curLetter));
      this.nextLetter = this.realPhrase.charAt(this.curLetter) + '[' + this.currentMorseHint + ']';
      clearTimeout(this.bulbOffTimer);
      this.bulbOffTimer = setTimeout(() => this.statusText = 'type', 1000);
    }
    else {
      this.nextLetter = '';
      this.showMorseTokenIdx = 0;
      this.currentMorseHint = '';
      this.showNextMorseToken();
    }    
  }

  showNextMorseToken() {
    this.bulbOn = true;
    const wait = this.morseCode[this.showMorseTokenIdx] ? 5*this.timeInterval : this.timeInterval;
    this.currentMorseHint += this.morseCode[this.showMorseTokenIdx] ? '-' : '.';
    if(this.morseCode[this.showMorseTokenIdx]) {
      window['badPractice'].dashSound.setVolume(2);
      window['badPractice'].dashSound.play();
    }
    else {
      window['badPractice'].dotSound.setVolume(2);
      window['badPractice'].dotSound.play();
    }
    clearTimeout(this.bulbOnTimer);
    this.bulbOnTimer = setTimeout(() => {
      this.bulbOff();
      this.statusText = 'type';
    }, wait);
  }

  bulbOff() {
    this.bulbOn = false;
    this.showMorseTokenIdx++;
    clearTimeout(this.bulbOffTimer);
    if(this.isNextSpace()) {
      //it is a space, so we go to the next word
      this.bulbOffTimer = setTimeout(() => this.letterGuessed(), 5*this.timeInterval);
    } else if (this.showMorseTokenIdx < this.morseCode.length) {
      this.bulbOffTimer = setTimeout(() => this.showNextMorseToken(), 5*this.timeInterval);
    }
  }

  startGuessing() {

    this.realPhrase = this.remotePhraseData.text;//this.getPhraseByIdx(phraseIdx);
    
    this.statusText = 'type';
    this.curLetter = 0;
    this.errors = 0;

    if(this.isSender) {
      this.phraseToGuess = this.realPhrase.substr(1);
      this.guessedPart = this.realPhrase.charAt(0);
    }
    else {
      this.phraseToGuess = this.realPhrase.replace(/[a-zA-Z0-9]/g,'_').substr(1);
      this.guessedPart = '_';
    }    

    this.showLetter();
  }

  letterError() {
    this.statusText = 'error';
    this.currentMorseHint = '';
    this.errors++;
    window['badPractice'].errorSound.play();
    this.showLetter();
  }

  get isError(): boolean {
    return this.statusText === 'error';
  }

  getSoundVar(err): number {
    err = Math.min(1, err / 20);

    if(Math.random() < 0.5) {
      return Math.max(0.1, 1 - err);
    }

    return Math.min(2, 1 + err);
  }

  phraseGuessed() {
    this.statusText = 'done!';
    this.currentMorseHint = 'done!';

    window['badPractice'].say(this.realPhrase, this.getSoundVar(this.remotePhraseData.senderErrors), this.getSoundVar(this.errors));

    if(!this.localMode && this.remoteData && this.isSender) {
      this.remoteData.set({
        text: this.realPhrase,
        senderErrors: this.errors
      });
    }
  }

  letterGuessed() {
    this.statusText = 'bingo';
    this.phraseToGuess = this.phraseToGuess.substr(1);
    if(this.curLetter + 1 >= this.realPhrase.length) {
      this.phraseGuessed();
      return;
    }
    this.guessedPart = this.guessedPart.substr(0, this.guessedPart.length - 1) + 
                        this.realPhrase.charAt(this.curLetter) + 
                        (this.isSender ? 
                          this.realPhrase.charAt(this.curLetter + 1) : 
                          (this.morseService.getMorseStringForLetter(this.realPhrase.charAt(this.curLetter + 1)) ? '_' : ' ')
                        );
    this.curLetter++;
    this.showLetter();
  }

  checkMorseCode() {
    let i = 0;
    for(i = 0; i < this.morseCode.length && i < this.currentMorseTyped.length; i++) {
      if((this.morseCode[i] === 1 && this.currentMorseTyped.charAt(i) !== '-') ||
        (this.morseCode[i] === 0 && this.currentMorseTyped.charAt(i) !== '.')) {
          this.letterError();
          return;
      }
    }

    if(this.morseCode.length == this.currentMorseTyped.length) {
      this.currentMorseHint = 'bingo';
      this.letterGuessed();
    }
  }

  get isTouchDevice() {
    return 'ontouchstart' in window        // works on most browsers 
        || 'onmsgesturechange' in window;  // works on IE10 with some false positives
  };

  onButtonDown() {
    this.lastDownEvent = (new Date()).getTime();
  }

  onButtonUp() {
    //window.badPractice.say('Say something, pleease...', 1, 1);
    if(!this.lastDownEvent) {
      return;
    }

    if((new Date()).getTime() - this.lastDownEvent > this.timeInterval) {
      this.currentMorseTyped += '-';
      //window.badPractice.dashSound.setVolume(2);
      //window.badPractice.dashSound.play();
    }
    else {
      this.currentMorseTyped += '.';
      //window.badPractice.dotSound.setVolume(2);
      //window.badPractice.dotSound.play();
    }
    this.lastDownEvent = 0;
    
    if (this.curLetter > -1) {
      this.checkMorseCode();
    }
  }

}
